"""Module containing strings used while communicating with the user
in the physbiblio package.

This module is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "common",
    "gui",
    "main",
    "webimport",
]
