"""The package that contains the tests for the `physbiblio.gui` subpackage.

This file is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "test_basicDialogs",
    "test_bibWindows",
    "test_catWindows",
    "test_commonClasses",
    "test_dialogWindows",
    "test_errorManager",
    "test_expWindows",
    "test_inspireStatsGUI",
    "test_mainWindow",
    "test_marks",
    "test_profilesManager",
    "test_threadElements",
]
