class CommonStrings:
    """Some common strings for the package"""

    abstract = "Abstract"
    andC = "AND"
    cancel = "Cancel"
    clean = "Clean"
    closing = "Closing..."
    close = "Close"
    default = "Default"
    delete = "Delete"
    deleteQ = "Delete?"
    description = "Description"
    doneD = "Done."
    doneDD = "...done!"
    doneE = "Done!"
    error = "Error"
    errorInvalidParent = "Cannot run %s: invalid parent"
    failedComplete = "Failed in completing info for entry '%s'\n"
    false = "False"
    filename = "Filename"
    information = "Information"
    load = "Load"
    merge = "Merge"
    modify = "Modify"
    noAttribute = "%s has no attribute '%s'"
    nothingFound = "Nothing found."
    ok = "OK"
    opening = "Opening '%s'..."
    openingFailed = "Opening %s for '%s' failed!"
    orC = "OR"
    order = "Order"
    overwrite = "Overwrite"
    question = "Question"
    ready = "Ready"
    replace = "Replace"
    save = "Save"
    shortName = "Short name"
    stop = "Stop"
    true = "True"
    types = "Types"
    warning = "Warning"
