"""Module for retrieving bibtex information
from a list of given web resources
(see `__all__`).

This module is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "adsnasa",
    "arxiv",
    "doi",
    "inspire",
    "webInterf",
    "tests",
]
